# Step to make a call to weaviate collection to fetch relevant chunks
import weaviate
from weaviate.classes.init import Auth
from weaviate.client import WeaviateClient
from weaviate.collections.collection import Collection
from typing import Union, Dict, Any
from athina.steps import Step
from jinja2 import Environment
import time


class WeaviateRetrieval(Step):
    """
    Step that makes a call to weaviate collection to fetch relevant chunks.

    Attributes:
    url: URL of the Weaviate instance.
    collection_name: Name of the Weaviate collection to query.
    key: Key to extract from the response objects.
    search_type: Type of search to perform (semantic_search, keyword_search, hybrid_search).
    limit: Maximum number of results to fetch.
    api_key: API key for the Weaviate server.
    openai_api_key: OpenAI Api Key.
    input_column: Column name in the input data.
    env: Jinja environment.
    """

    url: str
    collection_name: str
    key: str
    search_type: str
    limit: int
    api_key: str
    openai_api_key: str
    input_column: str
    env: Environment = None
    _client: WeaviateClient = None
    _collection: Collection = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._client = weaviate.connect_to_weaviate_cloud(
            cluster_url=self.url,
            auth_credentials=Auth.api_key(self.api_key),
            headers={"X-OpenAI-Api-Key": self.openai_api_key},
        )
        self._collection = self._client.collections.get(self.collection_name)

    class Config:
        arbitrary_types_allowed = True

    def execute(self, input_data: Any) -> Union[Dict[str, Any], None]:
        """Makes a call to weaviate collection to fetch relevant chunks"""
        start_time = time.perf_counter()

        if input_data is None:
            input_data = {}

        if not isinstance(input_data, dict):
            return self._create_step_result(
                status="error",
                data="Input data must be a dictionary.",
                start_time=start_time,
            )
        input_text = input_data.get(self.input_column, None)

        if input_text is None:
            return self._create_step_result(
                status="error",
                data="Input column not found.",
                start_time=start_time,
            )

        try:
            if self.search_type == "semantic_search":
                response = self._collection.query.near_text(
                    query=input_text, limit=self.limit
                )
            elif self.search_type == "keyword_search":
                response = self._collection.query.bm25(
                    query=input_text, limit=self.limit
                )
            elif self.search_type == "hybrid_search":
                response = self._collection.query.hybrid(
                    query=input_text, limit=self.limit
                )
            else:
                raise ValueError(f"Unsupported search type: {self.search_type}")

            results = []
            for obj in response.objects:
                results.append(obj.properties[self.key])

            return self._create_step_result(
                status="success",
                data=results,
                start_time=start_time,
            )
        except Exception as e:
            return self._create_step_result(
                status="error",
                data=str(e),
                start_time=start_time,
            )

    def close(self):
        """Closes the connection to the Weaviate client."""
        if self._client:
            self._client.close()
            self._client = None



# Guardrails Gibberish Evaluator
# https://hub.guardrailsai.com/validator/guardrails/gibberish_text

import time
from typing import Dict, List, Optional
from athina.helpers.logger import logger
from ...base_evaluator import BaseEvaluator
from athina.metrics.metric_type import MetricType
from athina.interfaces.result import EvalResult, EvalResultMetric


# Passes when the text is sensible, fails when the text is gibberish.
class NotGibberishText(BaseEvaluator):
    _validation_method: str
    _threshold: float

    def __init__(
        self,
        validation_method: str = "sentence",
        threshold: float = 0.75,
    ):
        from guardrails.hub import GibberishText as GuardrailsGibberishText

        self._validation_method = validation_method
        self._threshold = threshold
        # Initialize Validator
        self.validator = GuardrailsGibberishText(
            threshold=self._threshold,
            validation_method=self._validation_method,
            on_fail="noop",
        )

    @property
    def name(self) -> str:
        return "NotGibberishText"

    @property
    def display_name(self) -> str:
        return "Not Gibberish Text"

    @property
    def metric_ids(self) -> List[str]:
        return [MetricType.PASSED.value]

    @property
    def required_args(self) -> List[str]:
        return ["response"]  # TODO: allow running this on user_query OR response

    @property
    def examples(self):
        pass

    def to_config(self) -> Optional[Dict]:
        return None

    def is_failure(self, result: bool) -> bool:
        return not (bool(result))

    def _evaluate(self, **kwargs) -> EvalResult:
        """
        Run the Guardrails evaluator.
        """
        from guardrails import Guard

        start_time = time.time()
        self.validate_args(**kwargs)
        metrics = []
        try:
            text = kwargs["response"]
            # Setup Guard
            guard = Guard.from_string(validators=[self.validator])
            # Pass LLM output through guard
            guard_result = guard.parse(text)
            grade_reason = (
                "Text is sensible"
                if guard_result.validation_passed
                else "Text is gibberish"
            )
            # Boolean evaluator
            metrics.append(
                EvalResultMetric(
                    id=MetricType.PASSED.value,
                    value=float(guard_result.validation_passed),
                )
            )
        except Exception as e:
            logger.error(f"Error occurred during eval: {e}")
            raise e

        end_time = time.time()
        eval_runtime_ms = int((end_time - start_time) * 1000)
        llm_eval_result = EvalResult(
            name=self.name,
            display_name=self.display_name,
            data=kwargs,
            failure=self.is_failure(guard_result.validation_passed),
            reason=grade_reason,
            runtime=eval_runtime_ms,
            model=None,
            metrics=metrics,
        )
        return {k: v for k, v in llm_eval_result.items() if v is not None}



from typing import Union, Dict, Any, Optional
from athina.steps import Step
from llama_parse import LlamaParse
import nest_asyncio
import time

nest_asyncio.apply()  # LlamaParse can cause nested asyncio exceptions so we need this line of code


class ParseDocument(Step):
    """
    Step that uses the llama_parse package to extract text from various document formats.

    Attributes:
        file_url: The URL of the file to be parsed.
        output_format: The type of result to return. Options: 'text' or 'markdown'. Default is 'text'.
        llama_parse_key: The API key to use for the LlamaParse API.
        verbose: Whether to print verbose output. Default is False.
    """

    file_url: str
    output_format: Optional[str] = "text"
    llama_parse_key: str
    verbose: Optional[bool] = False

    def execute(self, input_data) -> Union[Dict[str, Any], None]:
        """Parse a document using LlamaParse and return the result."""
        start_time = time.perf_counter()

        if input_data is None:
            input_data = {}

        if not isinstance(input_data, dict):
            return self._create_step_result(
                status="error",
                data="Input data must be a dictionary.",
                start_time=start_time,
            )

        try:
            # Initialize LlamaParse client
            llama_parse = LlamaParse(
                api_key=self.llama_parse_key,
                verbose=self.verbose,
                result_type=self.output_format,
            )

            # Parse the document
            documents = llama_parse.load_data(file_path=self.file_url)

            if not documents:
                return self._create_step_result(
                    status="error",
                    data="No documents were parsed.",
                    start_time=start_time,
                )

            parsed_content = "\n".join(doc.text for doc in documents)

            return self._create_step_result(
                status="success",
                data=parsed_content,
                start_time=start_time,
            )

        except Exception as e:
            return self._create_step_result(
                status="error",
                data=f"LlamaParse error: {str(e)}",
                start_time=start_time,
            )



from ..example import FewShotExample, FewShotExampleInputParam

FAITHFULNESS_EVAL_EXAMPLES = [
    FewShotExample(
        input_params=[
            FewShotExampleInputParam(
                name="context",
                value="Y Combinator is a startup accelerator launched in March 2005. It has been used to launch more than 4,000 companies.",
            ),
            FewShotExampleInputParam(
                name="response",
                value="YC invests $125,000 in startups in exchange for equity.",
            ),
        ],
        eval_result="Fail",
        eval_reason="The response cannot be inferred from the provided context. The context does not mention that YC invests $125,000 in startups.",
    ),
    FewShotExample(
        input_params=[
            FewShotExampleInputParam(
                name="context",
                value="The president of the United States is Joe Biden.",
            ),
            FewShotExampleInputParam(
                name="response",
                value="Barack Obama was the 44th president of the United States.",
            ),
        ],
        eval_result="Fail",
        eval_reason="The response cannot be inferred from the provided context. The context does not state anything that suggests Barack Obama was the 44th president of the United States.",
    ),
]
